package ap.proof;

import ap.proof.goal.{PrioritisedTask, Goal, TaskManager}
import ap.terfor.{Formula, ConstantTerm, TermOrder}
import ap.terfor.conjunctions.{Conjunction, NegatedConjunctions, Quantifier}
import ap.terfor.substitutions.Substitution
import ap.parameters.GoalSettings

class SimpleProofTreeFactory(removeTask : Boolean,
                             simplifier : ConstraintSimplifier) extends ProofTreeFactory {

  def and(subtrees : Seq[ProofTree], order : TermOrder) : ProofTree =
    AndTree(subtrees, order, simplifier)
  
  def quantify(subtree : ProofTree,
               quan : Quantifier,
               quantifiedConstants : Set[ConstantTerm],
               order : TermOrder,
               subtreeOrder : TermOrder) : ProofTree =
    QuantifiedTree(subtree, quan, quantifiedConstants, order, subtreeOrder, simplifier)
  
  def substitute(subtree : ProofTree,
                 subst : Substitution,
                 order : TermOrder,
                 subtreeOrder : TermOrder) : ProofTree =
    SubstTree(subtree, subst, order, subtreeOrder, simplifier)
               
  def weaken(subtree : ProofTree,
             disjunct : Conjunction,
             order : TermOrder) : ProofTree =
    WeakenTree(subtree, disjunct, order)
               
  // not supposed to do anything when constructing proofs
  def eliminatedConstant(subtree : ProofTree,
                         c : ConstantTerm,
                         witness : (Substitution, TermOrder) => Substitution,
                         order : TermOrder) : ProofTree =
     subtree
    
  //////////////////////////////////////////////////////////////////////////////
  
  protected def createNewGoal(updatedFacts : Conjunction,
                              updatedClauses : NegatedConjunctions,
                              updatedElimConstants : Set[ConstantTerm],
                              updatedOrder : TermOrder,
                              newTasks : TaskManager,
                              age : Int,
                              settings : GoalSettings) : ProofTree =
    Goal(updatedFacts, updatedClauses, newTasks, age,
         updatedElimConstants, updatedOrder, settings)
    
  def updateGoal(updatedFacts : Conjunction,
                 updatedClauses : NegatedConjunctions,
                 updatedElimConstants : Set[ConstantTerm],
                 updatedOrder : TermOrder,
                 newTasks : Iterable[PrioritisedTask],
                 goal : Goal) : ProofTree = {
    val updatedTasks = (if (removeTask)
                          goal.tasks.removeFirst
                        else
                          goal.tasks) ++ newTasks
    createNewGoal(updatedFacts, updatedClauses, updatedElimConstants,
                  updatedOrder, updatedTasks, goal.age + 1, goal.settings)
  }
  
  def updateGoal(tasks : TaskManager, goal : Goal) : ProofTree =
    createNewGoal(goal.facts, goal.clauses,
                  goal.eliminatedConstants, goal.order, tasks, goal.age + 1,
                  goal.settings)

}
