package ap.proof;

import ap.terfor.{Formula, TermOrder}
import ap.terfor.conjunctions.Conjunction
import ap.util.Debug

object WeakenTree {
  
  private val AC = Debug.AC_PROOF_TREE

  def apply(subtree : ProofTree,
            disjunct : Conjunction,
            order : TermOrder) : ProofTree =
    if (disjunct.isFalse) {
      subtree
    } else subtree match {
      case subtree : WeakenTree =>
        WeakenTree(subtree.subtree,
                   Conjunction.disj(Array(subtree.disjunct, disjunct), order),
                   order)
      case _ => new WeakenTree(subtree, disjunct, order)
    }
  
}

/**
 * <code>ProofTreeOneChild</code> that weakens the closing constraint of its
 * <code>subtree</code> by disjunctively adding a formula
 */
class WeakenTree private (val subtree : ProofTree,
                          val disjunct : Conjunction,
                          protected val order : TermOrder)
      extends ProofTreeOneChild {

  protected val subtreeOrder : TermOrder = order
  
  //////////////////////////////////////////////////////////////////////////////
  Debug.assertCtor(WeakenTree.AC,
                   (order isSortingOf disjunct) &&
                   !subtree.isInstanceOf[WeakenTree])
  //////////////////////////////////////////////////////////////////////////////
  
  protected[proof] lazy val rawClosingConstraint : Conjunction =
    Conjunction.disj(Array(subtree.closingConstraint, disjunct), subtreeOrder)

  lazy val closingConstraint : Conjunction = rawClosingConstraint
    
  lazy val stepMeaningful : Boolean = (subtree.stepMeaningful && !disjunct.isTrue)

  def update(newSubtree : ProofTree) : ProofTree =
    if (subtree == newSubtree)
      this
    else
      WeakenTree(newSubtree, disjunct, order)
  
  //////////////////////////////////////////////////////////////////////////////

  override def toString : String = {
    "WeakenTree (" + disjunct + ") (" + closingConstraint +
      ") {\n" + subtree + "\n}"
  }

}
