package ap.proof.goal;

import ap.terfor.conjunctions.{Conjunction, Quantifier}
import ap.util.Debug
import ap.parameters.GoalSettings

object FormulaTask {

  private val AC = Debug.AC_COMPLEX_FORMULAS_TASK
  
}


/**
 * The representation of formulas in a proof goal that are more complex than
 * simple facts. Such formulas are considered to have positive
 * polarity, i.e., as conjunctions in the succedent of a goal.
 * This class is both responsible for storing such formulas and for eventually
 * processing the formulas, e.g. by splitting up the formulas/proof goal.
 */
abstract class FormulaTask(val formula : Conjunction, val age : Int)
         extends PrioritisedTask {

  //////////////////////////////////////////////////////////////////////////////
  Debug.assertCtor(FormulaTask.AC, isCoveredFormula(formula))
  //////////////////////////////////////////////////////////////////////////////

  /**
   * Return <code>true</code> if <code>f</code> is a formula that can be handled
   * by this task
   */
  protected def isCoveredFormula(f : Conjunction) : Boolean
  
  /**
   * Perform the actual task (whatever needs to be done with <code>formula</code>)
   */
  protected def doTask(goal : Goal, ptf : ProofTreeFactory) : ProofTree
  
  /**
   * Create a new <code>FormulaTask</code> by updating the value of
   * <code>formula</code>
   */
  protected def updateFormula(f : Conjunction, goal : Goal) : FormulaTask

  /**
   * Update the task with possibly new information from the goal
   */
  def updateTask(goal : Goal, factCollector : Conjunction => unit)
                                                   : Seq[FormulaTask] = {
    val reducedFormula = goal reduceWithFacts formula
    if (reducedFormula == formula) {
      List(this)
    } else if (isCoveredFormula(reducedFormula)) {
      List(this.updateFormula(reducedFormula, goal))
    } else {
      if (AddFactsTask isCoveredFormula reducedFormula) {
        factCollector(reducedFormula)
        List()
      } else {
        goal formulaTasks reducedFormula
      }
    }     
  }
  
  def apply(goal : Goal, ptf : ProofTreeFactory) : ProofTree = this.doTask(goal, ptf)

  protected val name : String
   
  override def toString : String = name + "(" + priority + ", " + formula + ")"
   
}
