package ap.terfor.equations;

import ap.basetypes.IdealInt
import ap.terfor.linearcombination.LinearCombination
import ap.util.Seqs

/**
 * Abstract class for solving a conjunction of equations through column
 * operations.
 */
abstract class ColumnSolver(equations : EquationConj, order : TermOrder) {

  private var curEqs = equations
  private var curOrder = order
  
  /**
   * Decide whether the given linear combination describes an equation that can
   * be solved/simplified through a column operation. This method has to be
   * implemented by subclasses and has to return <code>None</code> if the equation
   * cannot be simplified, otherwise a triplet consisting of the new "small symbol",
   * the term that the small symbol is supposed to represent, and the updated
   * <code>TermOrder</code>
   */
  protected def isSolvableEq(lc : LinearCombination, order : TermOrder)
                               : Option[(Term, LinearCombination, TermOrder)]

  lazy val result = {
    var cont : Boolean = true    
    while (cont && !curEqs.isFalse) {
      Seqs.some(for (lc <- curEqs.elements) yield isSolvableEq(lc, curOrder)) match {
        case Some((smallConst, definition, newOrder)) => {
          curOrder = newOrder
          val scLc = LinearCombination(smallConst, curOrder)
          val scDefLc = LinearCombination.sum(Array((IdealInt.ONE, definition),
                                                    (IdealInt.MINUS_ONE, scLc)),
                                              curOrder)
          val newEq = EquationConj(scDefLc, curOrder)
          curEqs = EquationConj.conj(Array(newEq, curEqs), curOrder)
        }
        case None => cont = false
      }
    }

    if (curEqs.isFalse) curOrder = TermOrder.EMPTY

    (curEqs, curOrder)
  }
}
